@@ -154,7 +154,7 @@ http1_wait(struct sess *sp, struct worker *wrk, struct req *req)
 			if (isnan(req->t_first))
 				/* Record first byte received time stamp */
 				req->t_first = now;
-			when = sp->t_idle + cache_param->timeout_req;
+			when = req->t_first + cache_param->timeout_req;
 			tmo = (int)(1e3 * (when - now));
 			if (when < now || tmo == 0) {
 				why = SC_RX_TIMEOUT;
@@ -376,6 +376,13 @@ http1_dissect(struct worker *wrk, struct req *req)
 	AZ(req->err_code);
 	req->ws_req = WS_Snapshot(req->ws);
 	req->doclose = http_DoConnection(req->http);
+	if (req->doclose == SC_RX_BAD) {
+		r = write(req->sp->fd, r_400, strlen(r_400));
+		if (r > 0)
+			req->acct.resp_hdrbytes += r;
+		SES_Close(req->sp, req->doclose);
+		return (REQ_FSM_DONE);
+	}
 
 	http_Unset(req->http, H_Expect);
 
