@@ -68,7 +68,7 @@ vrt_do_string(struct http *hp, int fld,
 
 #define VRT_HDR_L(obj, hdr, fld)					\
 void									\
-VRT_l_##obj##_##hdr(const struct vrt_ctx *ctx, const char *p, ...)	\
+VRT_l_##obj##_##hdr(VRT_CTX, const char *p, ...)	\
 {									\
 	va_list ap;							\
 									\
@@ -80,7 +80,7 @@ VRT_l_##obj##_##hdr(const struct vrt_ctx *ctx, const char *p, ...)	\
 
 #define VRT_HDR_R(obj, hdr, fld)					\
 const char *								\
-VRT_r_##obj##_##hdr(const struct vrt_ctx *ctx)				\
+VRT_r_##obj##_##hdr(VRT_CTX)				\
 {									\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);				\
 	CHECK_OBJ_NOTNULL(ctx->http_##obj, HTTP_MAGIC);			\
@@ -93,7 +93,7 @@ VRT_r_##obj##_##hdr(const struct vrt_ctx *ctx)				\
 
 #define VRT_STATUS_L(obj)						\
 void									\
-VRT_l_##obj##_status(const struct vrt_ctx *ctx, long num)		\
+VRT_l_##obj##_status(VRT_CTX, long num)		\
 {									\
 									\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);				\
@@ -112,7 +112,7 @@ VRT_l_##obj##_status(const struct vrt_ctx *ctx, long num)		\
 
 #define VRT_STATUS_R(obj)						\
 long									\
-VRT_r_##obj##_status(const struct vrt_ctx *ctx)				\
+VRT_r_##obj##_status(VRT_CTX)				\
 {									\
 									\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);				\
@@ -148,7 +148,7 @@ VRT_STATUS_R(beresp)
 #define VBERESPW0(field)
 #define VBERESPW1(field)						\
 void									\
-VRT_l_beresp_##field(const struct vrt_ctx *ctx, unsigned a)		\
+VRT_l_beresp_##field(VRT_CTX, unsigned a)		\
 {									\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);				\
 	CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);			\
@@ -158,7 +158,7 @@ VRT_l_beresp_##field(const struct vrt_ctx *ctx, unsigned a)		\
 #define VBERESPR0(field)
 #define VBERESPR1(field)						\
 unsigned								\
-VRT_r_beresp_##field(const struct vrt_ctx *ctx)				\
+VRT_r_beresp_##field(VRT_CTX)				\
 {									\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);				\
 	CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);			\
@@ -173,22 +173,8 @@ VRT_r_beresp_##field(const struct vrt_ctx *ctx)				\
 
 /*--------------------------------------------------------------------*/
 
-void
-VRT_l_bereq_uncacheable(const struct vrt_ctx *ctx, unsigned a)
-{
-	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
-
-	if (ctx->bo->do_pass && !a) {
-		VSLb(ctx->vsl, SLT_VCL_Error,
-		    "Ignoring attempt to reset bereq.uncacheable");
-	} else if (a) {
-		ctx->bo->do_pass = 1;
-	}
-}
-
 unsigned
-VRT_r_bereq_uncacheable(const struct vrt_ctx *ctx)
+VRT_r_bereq_uncacheable(VRT_CTX)
 {
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
@@ -196,7 +182,7 @@ VRT_r_bereq_uncacheable(const struct vrt_ctx *ctx)
 }
 
 void
-VRT_l_beresp_uncacheable(const struct vrt_ctx *ctx, unsigned a)
+VRT_l_beresp_uncacheable(VRT_CTX, unsigned a)
 {
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
@@ -210,7 +196,7 @@ VRT_l_beresp_uncacheable(const struct vrt_ctx *ctx, unsigned a)
 }
 
 unsigned
-VRT_r_beresp_uncacheable(const struct vrt_ctx *ctx)
+VRT_r_beresp_uncacheable(VRT_CTX)
 {
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
@@ -220,7 +206,7 @@ VRT_r_beresp_uncacheable(const struct vrt_ctx *ctx)
 /*--------------------------------------------------------------------*/
 
 const char *
-VRT_r_client_identity(const struct vrt_ctx *ctx)
+VRT_r_client_identity(VRT_CTX)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -232,7 +218,7 @@ VRT_r_client_identity(const struct vrt_ctx *ctx)
 }
 
 void
-VRT_l_client_identity(const struct vrt_ctx *ctx, const char *str, ...)
+VRT_l_client_identity(VRT_CTX, const char *str, ...)
 {
 	va_list ap;
 	const char *b;
@@ -254,7 +240,7 @@ VRT_l_client_identity(const struct vrt_ctx *ctx, const char *str, ...)
 
 #define BEREQ_TIMEOUT(which)					\
 void								\
-VRT_l_bereq_##which(const struct vrt_ctx *ctx, double num)	\
+VRT_l_bereq_##which(VRT_CTX, double num)	\
 {								\
 								\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);			\
@@ -263,7 +249,7 @@ VRT_l_bereq_##which(const struct vrt_ctx *ctx, double num)	\
 }								\
 								\
 double								\
-VRT_r_bereq_##which(const struct vrt_ctx *ctx)			\
+VRT_r_bereq_##which(VRT_CTX)			\
 {								\
 								\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);			\
@@ -278,7 +264,7 @@ BEREQ_TIMEOUT(between_bytes_timeout)
 /*--------------------------------------------------------------------*/
 
 const char *
-VRT_r_beresp_backend_name(const struct vrt_ctx *ctx)
+VRT_r_beresp_backend_name(VRT_CTX)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -293,7 +279,7 @@ VRT_r_beresp_backend_name(const struct vrt_ctx *ctx)
 }
 
 VCL_IP
-VRT_r_beresp_backend_ip(const struct vrt_ctx *ctx)
+VRT_r_beresp_backend_ip(VRT_CTX)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -308,7 +294,7 @@ VRT_r_beresp_backend_ip(const struct vrt_ctx *ctx)
 /*--------------------------------------------------------------------*/
 
 const char *
-VRT_r_beresp_storage_hint(const struct vrt_ctx *ctx)
+VRT_r_beresp_storage_hint(VRT_CTX)
 {
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
@@ -319,7 +305,7 @@ VRT_r_beresp_storage_hint(const struct vrt_ctx *ctx)
 }
 
 void
-VRT_l_beresp_storage_hint(const struct vrt_ctx *ctx, const char *str, ...)
+VRT_l_beresp_storage_hint(VRT_CTX, const char *str, ...)
 {
 	va_list ap;
 	const char *b;
@@ -342,7 +328,7 @@ VRT_l_beresp_storage_hint(const struct vrt_ctx *ctx, const char *str, ...)
 #define REQ_VAR_L(nm, elem, type,extra)					\
 									\
 void									\
-VRT_l_req_##nm(const struct vrt_ctx *ctx, type arg)			\
+VRT_l_req_##nm(VRT_CTX, type arg)			\
 {									\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);				\
 	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);				\
@@ -353,7 +339,7 @@ VRT_l_req_##nm(const struct vrt_ctx *ctx, type arg)			\
 #define REQ_VAR_R(nm, elem, type)					\
 									\
 type									\
-VRT_r_req_##nm(const struct vrt_ctx *ctx)				\
+VRT_r_req_##nm(VRT_CTX)				\
 {									\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);				\
 	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);				\
@@ -368,7 +354,7 @@ REQ_VAR_R(ttl, d_ttl, double)
 /*--------------------------------------------------------------------*/
 
 void
-VRT_l_bereq_backend(const struct vrt_ctx *ctx, struct director *be)
+VRT_l_bereq_backend(VRT_CTX, struct director *be)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -377,7 +363,7 @@ VRT_l_bereq_backend(const struct vrt_ctx *ctx, struct director *be)
 }
 
 struct director *
-VRT_r_bereq_backend(const struct vrt_ctx *ctx)
+VRT_r_bereq_backend(VRT_CTX)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -388,7 +374,7 @@ VRT_r_bereq_backend(const struct vrt_ctx *ctx)
 /*--------------------------------------------------------------------*/
 
 void
-VRT_l_req_esi(const struct vrt_ctx *ctx, unsigned process_esi)
+VRT_l_req_esi(VRT_CTX, unsigned process_esi)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -402,7 +388,7 @@ VRT_l_req_esi(const struct vrt_ctx *ctx, unsigned process_esi)
 }
 
 unsigned
-VRT_r_req_esi(const struct vrt_ctx *ctx)
+VRT_r_req_esi(VRT_CTX)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -411,7 +397,7 @@ VRT_r_req_esi(const struct vrt_ctx *ctx)
 }
 
 long
-VRT_r_req_esi_level(const struct vrt_ctx *ctx)
+VRT_r_req_esi_level(VRT_CTX)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -422,7 +408,7 @@ VRT_r_req_esi_level(const struct vrt_ctx *ctx)
 /*--------------------------------------------------------------------*/
 
 unsigned
-VRT_r_req_can_gzip(const struct vrt_ctx *ctx)
+VRT_r_req_can_gzip(VRT_CTX)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -433,7 +419,7 @@ VRT_r_req_can_gzip(const struct vrt_ctx *ctx)
 /*--------------------------------------------------------------------*/
 
 long
-VRT_r_req_restarts(const struct vrt_ctx *ctx)
+VRT_r_req_restarts(VRT_CTX)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -442,7 +428,7 @@ VRT_r_req_restarts(const struct vrt_ctx *ctx)
 }
 
 long
-VRT_r_bereq_retries(const struct vrt_ctx *ctx)
+VRT_r_bereq_retries(VRT_CTX)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -458,7 +444,7 @@ VRT_r_bereq_retries(const struct vrt_ctx *ctx)
 #define VRT_DO_EXP_L(which, sexp, fld)				\
 								\
 void								\
-VRT_l_##which##_##fld(const struct vrt_ctx *ctx, double a)	\
+VRT_l_##which##_##fld(VRT_CTX, double a)	\
 {								\
 								\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);			\
@@ -472,7 +458,7 @@ VRT_l_##which##_##fld(const struct vrt_ctx *ctx, double a)	\
 #define VRT_DO_EXP_R(which, sexp, fld, offset)			\
 								\
 double								\
-VRT_r_##which##_##fld(const struct vrt_ctx *ctx)		\
+VRT_r_##which##_##fld(VRT_CTX)		\
 {								\
 								\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);			\
@@ -498,7 +484,7 @@ VRT_DO_EXP_R(beresp, ctx->bo->exp, keep, 0)
  */
 
 const char *
-VRT_r_req_xid(const struct vrt_ctx *ctx)
+VRT_r_req_xid(VRT_CTX)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -509,7 +495,7 @@ VRT_r_req_xid(const struct vrt_ctx *ctx)
 }
 
 const char *
-VRT_r_bereq_xid(const struct vrt_ctx *ctx)
+VRT_r_bereq_xid(VRT_CTX)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -523,7 +509,7 @@ VRT_r_bereq_xid(const struct vrt_ctx *ctx)
 
 #define REQ_BOOL(hash_var)					\
 void								\
-VRT_l_req_##hash_var(const struct vrt_ctx *ctx, unsigned val)	\
+VRT_l_req_##hash_var(VRT_CTX, unsigned val)	\
 {								\
 								\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);			\
@@ -532,7 +518,7 @@ VRT_l_req_##hash_var(const struct vrt_ctx *ctx, unsigned val)	\
 }								\
 								\
 unsigned							\
-VRT_r_req_##hash_var(const struct vrt_ctx *ctx)			\
+VRT_r_req_##hash_var(VRT_CTX)			\
 {								\
 								\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);			\
@@ -546,7 +532,7 @@ REQ_BOOL(hash_always_miss)
 /*--------------------------------------------------------------------*/
 
 VCL_IP
-VRT_r_client_ip(const struct vrt_ctx *ctx)
+VRT_r_client_ip(VRT_CTX)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -556,7 +542,7 @@ VRT_r_client_ip(const struct vrt_ctx *ctx)
 }
 
 VCL_IP
-VRT_r_server_ip(const struct vrt_ctx *ctx)
+VRT_r_server_ip(VRT_CTX)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -566,7 +552,7 @@ VRT_r_server_ip(const struct vrt_ctx *ctx)
 }
 
 const char*
-VRT_r_server_identity(const struct vrt_ctx *ctx)
+VRT_r_server_identity(VRT_CTX)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -577,7 +563,7 @@ VRT_r_server_identity(const struct vrt_ctx *ctx)
 }
 
 const char*
-VRT_r_server_hostname(const struct vrt_ctx *ctx)
+VRT_r_server_hostname(VRT_CTX)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -589,18 +575,17 @@ VRT_r_server_hostname(const struct vrt_ctx *ctx)
 /*--------------------------------------------------------------------*/
 
 long
-VRT_r_obj_hits(const struct vrt_ctx *ctx)
+VRT_r_obj_hits(VRT_CTX)
 {
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->req->obj, OBJECT_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->req->obj->objcore, OBJCORE_MAGIC);
-	CHECK_OBJ_NOTNULL(ctx->req->obj->objcore->objhead, OBJHEAD_MAGIC);
-	return (ctx->req->obj->objcore->objhead->hits);
+	return (ctx->req->obj->objcore->hits);
 }
 
 unsigned
-VRT_r_obj_uncacheable(const struct vrt_ctx *ctx)
+VRT_r_obj_uncacheable(VRT_CTX)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -613,7 +598,7 @@ VRT_r_obj_uncacheable(const struct vrt_ctx *ctx)
 
 #define HTTP_VAR(x)						\
 struct http *							\
-VRT_r_##x(const struct vrt_ctx *ctx)				\
+VRT_r_##x(VRT_CTX)				\
 {								\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);			\
 	CHECK_OBJ_NOTNULL(ctx->http_##x, HTTP_MAGIC);		\
