@@ -73,12 +73,12 @@ struct vtc_job {
 };
 
 int iflg = 0;
+unsigned vtc_maxdur = 60;
 
 static VTAILQ_HEAD(, vtc_tst) tst_head = VTAILQ_HEAD_INITIALIZER(tst_head);
 static struct vev_base *vb;
 static int njob = 0;
 static int npar = 1;			/* Number of parallel tests */
-static unsigned vtc_maxdur = 60;	/* Max duration of any test */
 static int vtc_continue;		/* Continue on error */
 static int vtc_verbosity = 1;		/* Verbosity Level */
 static int vtc_good;
@@ -260,7 +260,7 @@ start_test(void)
 	ALLOC_OBJ(jp, JOB_MAGIC);
 	AN(jp);
 
-	jp->bufsiz = 256*1024;		/* XXX */
+	jp->bufsiz = 512*1024;		/* XXX */
 
 	jp->buf = mmap(NULL, jp->bufsiz, PROT_READ|PROT_WRITE,
 	    MAP_ANON | MAP_SHARED, -1, 0);
