@@ -29,14 +29,16 @@
 #include "config.h"
 
 #include <sys/types.h>
+
+#include <errno.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <unistd.h>
 
+#include "vas.h"
 #include "vcli.h"
-#include "libvarnish.h"
 #include "vsha256.h"
 
-
 void
 VCLI_AuthResponse(int S_fd, const char *challenge,
     char response[CLI_AUTH_RESPONSE_LEN + 1])
